#include "ext_eeprom.h"


void ext_eeprom_init (void)
{
	/*setting for 8MHz --> calculation: datasheet page 166
	EEPROM is specified up to 1MHz (SCL), I will use 250KHz
	Prescaler Bits TWPS0:1 in TWSR are unnecessary --> default: Prescaler=1
	SCL = CPU/(16+2*TWBR*4)*/
	TWBR = 0x01;
}


unsigned char ext_eeprom_send_start (void)
{
	/*writing a one to TWINT clears it, TWSTA=Start, TWEN=TWI-enable*/
	TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);
	/*wait, until start condition has been sent --> ACK*/
	while (!(TWCR & (1<<TWINT)));
	return TWSR;
}


void ext_eeprom_send_stop (void)
{
	/*writing a one to TWINT clears it, TWSTO=Stop, TWEN=TWI-enable*/
	TWCR = (1<<TWINT) | (1<<TWSTO) | (1<<TWEN);	
}


unsigned char ext_eeprom_send_add_rw (unsigned char address, unsigned char rw)
{
	/*address can be 0 .. 3; rw=0 --> write, rw=1 --> read*/
	unsigned char addr_byte = 0;
	/*shift address one bit left*/
	addr_byte = address << 1;
	/*set RW-Bit, if necessary*/
	addr_byte |= rw;
	/*0b10100xxx --> address of EEPROM*/
	addr_byte |= 0xA0;
	/*TWDR contains byte to send*/
	TWDR = addr_byte;
	/*send content of TWDR*/
	TWCR = (1<<TWINT) | (1<<TWEN);
	/*wait, until address has been sent --> ACK*/
	while (!(TWCR & (1<<TWINT)));
	return TWSR;
}


unsigned char ext_eeprom_send_byte (unsigned char byte)
{
	/*TWDR contains byte to send*/
	TWDR = byte;
	/*send content of TWDR*/
	TWCR = (1<<TWINT) | (1<<TWEN);
	/*wait, until byte has been sent --> ACK*/
	while (!(TWCR & (1<<TWINT)));
	return TWSR;
}


unsigned char ext_eeprom_read_byte (void)
{
	/*send content of TWDR; TWEA = enable ACK*/
	TWCR = (1<<TWINT) | (1<<TWEA) | (1<<TWEN);
	/*wait, until byte has been received --> ACK*/
	while (!(TWCR & (1<<TWINT)));
	return TWDR;
}


void ext_eeprom_send_page (unsigned char *array, unsigned short page, unsigned char deviceadd)
{
	/*initialization*/
	ext_eeprom_init ();

	/*start condition*/
	ext_eeprom_send_start ();

	/*send device address an R/W*/
	ext_eeprom_send_add_rw (deviceadd, 0);

	/*calculate adress*/
	unsigned short address = 128*page;

	/*send address MSB*/
	ext_eeprom_send_byte ((unsigned char) address >> 8);

	/*send address LSB*/
	ext_eeprom_send_byte ((unsigned char) address);

	/*write number of bytes in page*/
	for (unsigned char i=0; i<128; i++)
	{
		ext_eeprom_send_byte (array [i]);
	}
	/*send stop condition*/
	ext_eeprom_send_stop ();
}


void ext_eeprom_read_page (unsigned char *array, unsigned short page, unsigned char deviceadd)
{
	/*initialization*/
	ext_eeprom_init ();

	/*start condition*/
	ext_eeprom_send_start ();

	/*send device address an R/W*/
	ext_eeprom_send_add_rw (deviceadd, 0);

	/*calculate adress
	+127: because the EEPROM increments the Write-Pointer*/
	unsigned short address = 128*page+127;

	/*send address MSB*/
	ext_eeprom_send_byte ((unsigned char) address >> 8);

	/*send address LSB*/
	ext_eeprom_send_byte ((unsigned char) address);

	/*write dummy-byte in address*/
	ext_eeprom_send_byte (0x00);

	/*send start condition once again*/
	ext_eeprom_send_start ();

	/*send device address an R/W*/
	ext_eeprom_send_add_rw (deviceadd, 1);

	/*get page*/
	for (unsigned char i=0; i<128; i++)
	{
		unsigned char byte = ext_eeprom_read_byte ();
		array [i] = byte;		
	}
	/*send stop condition*/
	ext_eeprom_send_stop ();
	/*everything alright :-)*/
}

unsigned char ext_eeprom_poll_ack (unsigned char deviceadd)
{
	/*initialization*/
	ext_eeprom_init ();

	/*start condition*/
	ext_eeprom_send_start ();

	/*send device address an R/W*/
	if (ext_eeprom_send_add_rw (deviceadd, 0) == 0x18)
	{
		ext_eeprom_send_stop ();
		return 1;
	}
	else
	{
		ext_eeprom_send_stop ();
		return 0;
	}
}


